/* 
 * Dock.m created by root on 2003-05-03 21:59:34 +0000
 *
 * Project GSDock
 *
 * Created with ProjectCenter - http://www.gnustep.org
 *
 * $Id: Dock.m,v 1.30 2003/07/06 01:10:29 cehardin Exp $
 */

#include "Dock.h"

@implementation Dock

- (NSMenu*) menuForEvent:(NSEvent*) theEvent {
    NSLog(@"Dock:menuForEvent");
    NSPoint location = [theEvent locationInWindow];
    DockTileCell* theTile = [self tileAtPoint: location];
    NSMenu *menu = [[NSMenu alloc] initWithTitle:[theTile path]];
    NSMenuItem* item;
    
    if( [theTile tileType] == DTLogo) {
        item = [menu addItemWithTitle:@"Quit GSDock" 
            action:@selector(quitDockItemSelected:)
            keyEquivalent:@""];
            [item setTarget: theTile];
    }
    else if( [theTile tileType] == DTRecycler) {
        item = [menu addItemWithTitle:@"Empty" 
            action:@selector(emptyMenuItemSelected:)
            keyEquivalent:@""];
            [item setTarget: theTile];
    }
    else if( [theTile tileType] == DTApplication) {
        if( [theTile tileStatus] == DTLaunched) {
            item = [menu addItemWithTitle:@"Keep" 
                action:@selector(keepMenuItemSelected:)
                keyEquivalent:@""];
            [item setState: [theTile keep]];
            [item setTarget: theTile];
        }
        item = [menu addItemWithTitle:@"Show Location" 
            action:@selector(showLocationItemSelected:)
            keyEquivalent:@""];
            [item setTarget: theTile];
        
        if( [theTile tileStatus] == DTLaunched) {
            item = [menu addItemWithTitle:@"Quit" 
                action:@selector(keepMenuItemSelected:)
                keyEquivalent:@""];
            [item setTarget: theTile];
        }
    }
    else {
        return nil;
    }
    return menu;
}



- (BOOL) isFlipped
{
    #ifdef GNUSTEP
    return YES;
    #endif
    return NO;
}

- initWithFrame:(NSRect) rect
{
    NSLog(@"Dock initWindFrame:(NSRect) rect");
    dragSessionInProgress = NO;
    appsDragged = NO;
    mixedDragged = NO;
    filesDragged = nil;
    
    self = [super initWithFrame: rect 
	mode: NSTrackModeMatrix 
	cellClass: [DockTileCell class]
	numberOfRows: 0
	numberOfColumns: 0];

    if( !self) {
    	return nil;
    }
    
    [self registerDragAndDrop];
    
    [self setCellSize: NSMakeSize(64, 64)];

    [self setIntercellSpacing: NSMakeSize(0, 0)];
    
    [self setTarget:self];
    [self setDoubleAction: @selector(doubleClicked:)];

    [self setCellSize: NSMakeSize(64, 64)];    
    //read in defaults and create tiles from last time
    
    return self;
}

- (void )registerDragAndDrop 
{
    NSArray *draggedTypes;
    
    draggedTypes = [NSArray arrayWithObjects: NSFilenamesPboardType, nil];
    [self registerForDraggedTypes: draggedTypes];
    NSLog(@"REgistered for Dragged Types");
}


- (void) doubleClicked: (id) sender
{
    DockTileCell *clickedCell;
    NSLog(@"Double clicked!");
    clickedCell = [self selectedCell];
    [clickedCell doubleClicked];
}


- (DockTileCell*) insertTile:(int) number
{
    NSLog(@"insertTile");
    DockTileCell *tile;
    
    [self insertRow: number];

    tile = [self cellAtRow: number
	column: 0];

    [self resizeDock];
    
    return tile;
}


- (void) resizeDock
{
    NSLog(@"Dock resizeDock");
    NSRect screenRect;
    NSRect frameRect;
    NSSize cellSize;

    //Has the dock grown to large?  Need to make the tiles smaller?
    screenRect = [[NSScreen mainScreen] frame];
    if( ([self numberOfRows] * 64) > screenRect.size.height) {
	NSLog(@"Dock needs to shrink");
	cellSize.height = (float)screenRect.size.height / (float)[self numberOfRows];
	cellSize.width = cellSize.height;	
    }
    else {
	NSLog(@"Dock does not need to shrink");
	cellSize.height = 64;
	cellSize.width = 64;
    }


    //resize and move the dock, centering it horizontally on the right edge of screen
    frameRect.origin.x = screenRect.size.width - cellSize.width;  
    if( cellSize.height == 64) {
	frameRect.origin.y = ((screenRect.size.height - (cellSize.height * [self numberOfTiles])) / 2);
    }
    else {
	frameRect.origin.y = 0;
    }
    frameRect.size.width = cellSize.width;
    frameRect.size.height = cellSize.height * [self numberOfRows];

    [[self window] setFrame: frameRect display:NO animate:NO];
    [self setCellSize: cellSize];
    //[self setNeedsDisplay: YES];
    //[self selectAll: self];
}

- (int)numberOfTiles
{
    int row, column;
    [self getNumberOfRows: &row columns:&column];
    return row;
}


- (void)addLogoTile
{
    NSLog(@"addLogoTile");
    DockTileCell *tile;
    
    tile = [self insertTile: 1];
    
    [tile setImage: [NSImage imageNamed:@"DockLogo"]];
    [tile setPath: @"NONE"];
    [tile setTileType: DTLogo];
}

- (void)addRecyclerTile
{
    NSLog(@"addRecyclerTile");
    DockTileCell *tile;
    
    tile = [self insertTile: 0];
    
    [tile setImage: [NSImage imageNamed:@"Recycler"]];
    [tile setPath: @"NONE"];
    [tile setTileType: DTRecycler];
}

- (DockTileCell*)getTile:(NSString*) path
{
    int n;
    
    for(n = 0; n < [self numberOfTiles]; n++) {
	DockTileCell *tile;
	
	tile = [self cellAtRow:n column:0];
	if([[tile path] isEqualToString: path]) {
	    return tile;
	}
    }
    return nil;
}



- (void)addApplicationTileAtIndex: (int)place withPath:(NSString*)path withPID:(NSNumber*)pid withStatus:(DockTileStatus) status
{
    NSLog(@"addTile");
    DockTileCell *tile;
    NSString *appName;
    NSString *type;
    
    
    
    if( place >= ([self numberOfRows])) {
	NSLog(@"Can't put launcherTile in last row or beyond, the Logo goes there!");
	return;
    }
    
    if( place == 0) {
	NSLog(@"Can't put launcherTile in location 0, the recycler goes there!");
	return;
    }
    
    if( place < 0) {
	NSLog(@"Can't put launcherTile in a negative location, silly!");
	return;
    }
    
    NSWorkspace *workspace = [NSWorkspace sharedWorkspace];
    if( [workspace getInfoForFile: path application:&appName type:&type] ) {
        //is it already on the dock?
        tile = [self getTile: path];
        if( tile != nil) {
            [tile setTileStatus: DTLaunching];
        }
        else {
            tile = [self insertTile: place];
            [tile setPath: path];
            [tile setImage: [workspace iconForFile: path]];
            [tile setTileType: DTApplication];
            [tile setTileStatus: status];
        }
        return;
    }
    NSLog(@"That Path does not exist!");
}


//add an application tile right above the recycler
- (void)addApplicationTile: (NSString*)path withPID:(NSNumber*)pid withStatus:(DockTileStatus) status  
{
    [self addApplicationTileAtIndex: 1 withPath:path withPID:pid withStatus: status];
}


- (BOOL)removeApplicationTile:(NSString*)path
{
    int n;
    for(n = 0; n < [self numberOfRows]; n++) {
	DockTileCell *tile;
	
	tile = [self cellAtRow:n column:0];
	if([path isEqualToString: [tile path]] ) {
	    [self removeTileAtIndex: n];
	    return YES;
	}
    }
    return NO;
}

- (void) applicationTerminated:(NSString*) appPath {
    int n;
    for(n = 0; n < [self numberOfRows]; n++) {
	DockTileCell *tile;
	
	tile = [self cellAtRow:n column:0];
	if([appPath isEqualToString: [tile path]] ) {
            if( [tile keep] == NO) {
                [self removeTileAtIndex: n];
	    }
            else {
                [tile setTileStatus: DTNotRunning];
            }
	}
    }
}



- (void)removeTileAtIndex: (int)place {
    NSLog(@"removeTile");
    if( place >= [self numberOfRows]) {
	NSLog(@"Can't remove that tile cause it doesnt exist!");
	return;
    }
    
    if( place < 0) {
	NSLog(@"Can't remove Tile in a negative location, silly!");
	return;
    }
    
    
    [self removeRow: place];

    [self resizeDock];
}

- (void)removeTile: (DockTileCell*) tile {
    int row, column;
    
    [self getRow: &row
        column: &column
        ofCell: tile];
    
    [self removeTileAtIndex: row];
}



//Used for dragging to see if there are a mixture of file types
//in the drag session
//mixtures can ONLY be put in the recycler
- (BOOL) areFilesMixed: (NSArray*) files {
    //if there's only 1 file it's simple
    if( [files count] == 1) {
        return NO;
    }
    else {
        //is it mixed with apps and documents (other things too)?
        NSWorkspace *workspace = [NSWorkspace sharedWorkspace];
        NSString *fileType, *firstFileType;
        NSString *file, *appName;
        NSEnumerator *enumerator = [files objectEnumerator];
        file = [enumerator nextObject];
        
        [workspace getInfoForFile: file
            application: &appName
            type: &firstFileType];
        
        while( nil != (file = [enumerator nextObject])) {
            [workspace getInfoForFile: file
                application: &appName
                type: &fileType];
            
            if( [firstFileType isEqualToString: fileType] == NO ) {
                NSLog(@"File types are mixed");
                 return YES;
            }
        }
        NSLog(@"file types are the same");
        return NO;
    }
}


- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>) sender 
{
    NSPoint draggingLocation = [self convertPoint:[sender draggingLocation] fromView:nil];
    //NSPoint draggingLocation = [sender draggingLocation];
    printf("Dragging Entered At:\t %f, %f\n", draggingLocation.x, draggingLocation.y);
    
    
    //find out what is being dragged
    //we only accect files
    NSPasteboard *pasteboard;
    NSDragOperation sourceDragMask;
   
    
    //find out what is being dragged
    pasteboard = [sender draggingPasteboard];
    sourceDragMask = [sender draggingSourceOperationMask];
    
    #ifdef GNUSTEP
        //GNUstep way...
        NSData *pbData = [pasteboard dataForType:NSFilenamesPboardType];
        filesDragged = [NSUnarchiver unarchiveObjectWithData: pbData];
    #else
        //Cocoa way...
        filesDragged = [pasteboard propertyListForType:NSFilenamesPboardType];
    #endif
    
    [filesDragged retain];
    
    if(filesDragged == nil) {
	NSLog(@"NSUnarchiver returned nil!");
	return NSDragOperationNone;
    }
    //we are dealing with three possibilites:
    //a mixed type of stuff being dragged
    //app(s) being dragged, we make tiles for it
    //and "other", which we see if the tile can launch
    if( [self areFilesMixed: filesDragged]) {
        mixedDragged = YES;
    }
    else {
        NSWorkspace *workspace = [NSWorkspace sharedWorkspace];
        NSString *fileType, *appName;
        
        [workspace getInfoForFile: [filesDragged objectAtIndex: 0]
            application: &appName
            type: &fileType];
        
        if( [fileType isEqualToString: NSApplicationFileType] ) {
            appsDragged = YES;
        }
    }
}



- (void)draggingExited:(id <NSDraggingInfo>) sender 
{    
    NSPoint draggingLocation = [self convertPoint:[sender draggingLocation] fromView:nil];
    //NSPoint draggingLocation = [sender draggingLocation];
    printf("Dragging Exited At:\t %f, %f\n", draggingLocation.x, draggingLocation.y);

            
    //we are dealing with three possibilites:
    //a mixed type of stuff being dragged, which can only go in the recycler
    //app(s) being dragged, we make tiles for it
    //and "other", which we see if the tile can launch and request it to launch
    if( tileDraggingTo != nil) {
	[tileDraggingTo setHighlighted:NO];
        if( appsDragged == YES) {
            //ok, so there is a tile there already, it's gotta go
            [self removeTile: tileDraggingTo];
        }
    }
    dragSessionInProgress = NO;
    tileDraggingTo = nil;
    appsDragged = NO;
    mixedDragged = NO;
    [filesDragged release];
    filesDragged = nil;
}



-(BOOL)prepareforDragOperation:(id <NSDraggingInfo>) sender
{
    return YES;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>) sender 
{
	NSLog(@"Perform DragOperation");
        if( tileDraggingTo == nil) {
            dragSessionInProgress = NO;
            tileDraggingTo = nil;
            appsDragged = NO;
            mixedDragged = NO;
            [filesDragged release];
            filesDragged = nil;
            return NO;
        }
        if( [tileDraggingTo tileType] ==DTRecycler) {
            NSLog(@"Recycler is not implemented yet!");
            dragSessionInProgress = NO;
            tileDraggingTo = nil;
            appsDragged = NO;
            mixedDragged = NO;
            [filesDragged release];
            filesDragged = nil;
	    [tileDraggingTo setHighlighted:NO];
	    [self setNeedsDisplay:YES];
            return YES;
        }
        
        
        [tileDraggingTo setHighlighted: NO];
        [tileDraggingTo setKeep:YES];
        dragSessionInProgress = NO;
        tileDraggingTo = nil;
        appsDragged = NO;
        mixedDragged = NO;
        [filesDragged release];
        filesDragged = nil;
	[self setNeedsDisplay:YES];
        return YES;        

}



- (DockTileCell*) tileAtPoint:(NSPoint)thePoint {
    int row, column;
    
    [self getRow: &row
        column: &column
        forPoint: thePoint];
    
    return [self cellAtRow: row column: column];
}



//If we get here then we know that a good drag session is in
//progress (only files)
- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>) sender 
{
    //find out what is being dragged
    NSDragOperation sourceDragMask;
    
            
        
    //find out where it is and what cell it is in if at all
    //NSPoint draggingLocation = [self convertPoint:[sender draggingLocation] fromView:nil];
    NSPoint draggingLocation = [sender draggingLocation];
    printf("Dragging Updated At:\t %f, %f\n", draggingLocation.x, draggingLocation.y);
    
    int row, column;
    DockTileCell *theCell;
    theCell = [self tileAtPoint:draggingLocation];

    if( theCell == nil) {
	NSLog(@"that drogged point is not in a cell!");
        return NSDragOperationNone;
    }
    
    //we handle apps differently so lets deal with that first
    if( appsDragged == YES ) {
        //is it in the Logo Tile?
        if([theCell tileType] == DTLogo) {
            if( tileDraggingTo != theCell ) {
                if( tileDraggingTo != nil ) {
                    [tileDraggingTo setHighlighted:NO];
                    if( [tileDraggingTo tileType] != DTRecycler ) {
                        [self removeTile: tileDraggingTo];
                    }
                }
            } 
            tileDraggingTo = nil;
            //[self setNeedsDisplay:YES];
            return NSDragOperationNone;
        }
        //is it in the Recycler?
        if([theCell tileType] == DTRecycler) {
            if( tileDraggingTo != theCell ) {
                if( tileDraggingTo != nil ) {
                    [tileDraggingTo setHighlighted:NO];
                    [self removeTile: tileDraggingTo];
                    tileDraggingTo = theCell;
                    [tileDraggingTo setHighlighted:YES];
                    //[self setNeedsDisplay:YES];
                }
            } 
            //[self setNeedsDisplay:YES];
            return NSDragOperationDelete;
        }
        {
            //if we get here it must be somewhere else on the dock.
            //try to figure out the spot where the insertion will occur
            
            //do we have this tile already?
            if( nil != [self getTile: [filesDragged objectAtIndex:0]] ) {
                //of course if we are dragging it around we will have it already
                //so is it a ghost or one that was added beforehand
                if( [[self getTile: [filesDragged objectAtIndex:0]] isHighlighted] == NO) {
                    return NSDragOperationNone;
                }
            }
            int rowToInsertTo;
                                    
            rowToInsertTo = draggingLocation.y / [self cellSize].height;
            
            //see if we have started already
            if( tileDraggingTo == nil) { //this is a fresh drag (or a move from the logo)
                tileDraggingTo = [self insertTile: rowToInsertTo];
                [tileDraggingTo setTileType: DTApplication];
                [tileDraggingTo setTileStatus: DTNotRunning];
                [tileDraggingTo setHighlighted: YES];
                [tileDraggingTo setPath: [filesDragged objectAtIndex: 0]];
                [tileDraggingTo setImage: [[NSWorkspace sharedWorkspace] iconForFile: [filesDragged objectAtIndex: 0]]];
                //[self setNeedsDisplay:YES];
                return NSDragOperationLink;
            }
            else { //gotta see if we moved outta a different area
                if( tileDraggingTo == [self cellAtRow: rowToInsertTo column: 0] ) {
                    //nothing to do
                    //[self setNeedsDisplay:YES];
                    return NSDragOperationLink;
                }
                else { //we are being moved around
                    //gotta swap delete the old cell and make a new one, unless it was in the recylcer or Logo, of course
                    if( ([tileDraggingTo tileType] == DTRecycler) || ([tileDraggingTo tileType] == DTLogo) ) {
                        [tileDraggingTo setHighlighted:NO];
                    }
                    else {
                        [self removeTile: tileDraggingTo];
                    }
                    tileDraggingTo = [self insertTile: rowToInsertTo];
                    [tileDraggingTo setTileType: DTApplication];
                    [tileDraggingTo setTileStatus: DTNotRunning];
                    [tileDraggingTo setPath: [filesDragged objectAtIndex: 0]];
                    [tileDraggingTo setImage: [[NSWorkspace sharedWorkspace] iconForFile: [filesDragged objectAtIndex: 0]]];
                    [tileDraggingTo setHighlighted: YES];
                    //[self setNeedsDisplay:YES];
                    return NSDragOperationLink;
                    
                }	
            }
        }
    }  //HANDLED APP DRAGS

    else if(mixedDragged == YES) { //Mixtures of Apps and other thingg.  They can only go in the recycler
        //is it in the Recycler?
        if([theCell tileType] == DTRecycler) {
            if( tileDraggingTo != theCell ) {
                [theCell setHighlighted:YES];
                if( tileDraggingTo != nil ) {
                    [tileDraggingTo setHighlighted:NO];
                }
                tileDraggingTo = theCell;
            } 
            [self setNeedsDisplay:YES];
            return NSDragOperationDelete;
        }
        else {
            //if we get here it must be somewhere else on the dock.
            //Mixed drags can only go to the Recycler
            if( tileDraggingTo != nil ) {
                    [tileDraggingTo setHighlighted:NO];
                    tileDraggingTo = nil;
                    [self setNeedsDisplay:YES];
            }
            return NSDragOperationNone;
        }
    }  //HANDLED MIXED
    
    else {  //Handle everything else, mainly documents!
      //is it in the Logo Tile?
        if([theCell tileType] == DTLogo) {
            if( tileDraggingTo != theCell ) {
                if( tileDraggingTo != nil ) {
                    [tileDraggingTo setHighlighted:NO];
                }
            } 
            tileDraggingTo = nil;
            [self setNeedsDisplay:YES];
            return NSDragOperationNone;
        }
        //is it in the Recycler?
        if([theCell tileType] == DTRecycler) {
            if( tileDraggingTo != theCell ) {
                if( tileDraggingTo != nil ) {
                    [tileDraggingTo setHighlighted:NO];
                    tileDraggingTo = theCell;
                    [tileDraggingTo setHighlighted:YES];
                    [self setNeedsDisplay:YES];
                }
            } 
            [self setNeedsDisplay:YES];
            return NSDragOperationDelete;
        }
        //if we get here it must be on an appIcon
        if(tileDraggingTo != theCell) {
                if(tileDraggingTo != nil) {
                    [tileDraggingTo setHighlighted:NO];
                }
                //figure our if the app will accept the document
                if( [theCell doesAcceptFiles: filesDragged] ) {
                    tileDraggingTo = theCell;
                    [tileDraggingTo setHighlighted:YES];
                    [self setNeedsDisplay:YES];
                    return NSDragOperationGeneric;
                }
                else {
                    tileDraggingTo = nil;
                    return NSDragOperationNone;
                }
        }
        else { //if we get here we have already figure out the tile does accept this file
            return NSDragOperationGeneric;
        }
        
    }  //HANDLED DOC (EVERYTHING ELSE) DRAGS
  
    NSLog(@"draggingUpdated is confused!");
}



@end
