/* 
 * DockTile.m created by root on 2003-05-03 21:59:34 +0000
 *
 * Project GSDock
 *
 * Created with ProjectCenter - http://www.gnustep.org
 *
 * $Id: DockTileCell.m,v 1.24 2003/07/06 01:10:29 cehardin Exp $
 */

#include "DockTileCell.h"

@implementation DockTileCell

- init
{
    if (self = [super init]) {
	tileImage = [NSImage imageNamed:@"Tile"];
        tileHLImage = [NSImage imageNamed:@"TileHL"];
        tileNonRunningImage = [NSImage imageNamed:@"TileNonRunning"];
        tileNonRunningHLImage = [NSImage imageNamed:@"TileNonRunningHL"];
        
	[tileImage setScalesWhenResized:YES];
        [tileHLImage setScalesWhenResized:YES];
	[tileNonRunningImage setScalesWhenResized:YES];
        [tileNonRunningHLImage setScalesWhenResized:YES];
        
        [tileImage retain];
        [tileHLImage retain];
        [tileNonRunningImage retain];
        [tileNonRunningHLImage retain];
        
        iconSize = 0.75;  //default (and max as well) size is 75% of the tile
    }
    return self;
}

- (void) doubleClicked
{
    NSLog(@"DockTileCell got double click!");
    if( _type == DTLogo) {
	[[NSApplication sharedApplication] orderFrontStandardAboutPanel:self];
	return;
    }
    if(_type == DTRecycler) {
	return;
    }
    if(_type == DTApplication) {
	NSWorkspace *workspace;
	NSString *appName;
	NSString *type;
	
	workspace = [NSWorkspace sharedWorkspace];
	if( [workspace getInfoForFile: _path application:&appName type:&type] ) {
	    if( [type isEqualToString: NSPlainFileType] ) {
		[workspace openFile: _path];
	    }
	    if( [type isEqualToString: NSDirectoryFileType] ) {
		NSLog(@"Can't open NSDirectoryFileType!");
		return;
	    }
	    if( [type isEqualToString: NSApplicationFileType] ) {
                NSLog(@"REquest NSWorkspace to launch app!");
                [workspace launchApplication: _path];
                if( [self tileStatus] == DTNotRunning ) {
                    //at leat in cocoa we don't get a AppWillLaunchNotification
                    //so we set our status here
                    [self setTileStatus: DTLaunching];
                }
		return;
	    }
	    if( [type isEqualToString: NSFilesystemFileType] ) {
	    	NSLog(@"Can't open NSFilesystemFileType!");
		return;
	    }
	    if( [type isEqualToString: NSShellCommandFileType] ) {
	    	NSLog(@"Can't open NSShellCommandFileType!");
		return;
	    }
	}
	NSLog(@"I could not find that file!");
	return;
    }	
}


- (void) setPath: (NSString*) path 
{
    _path = path;
    [path retain];
}

- (NSString*) path 
{
    return _path;
}

- (DockTileType) tileType {
    return _type;
}

- (void) setTileType: (DockTileType) type {
    _type = type;
}


- (BOOL) keep
{
    return _keep;
}

- (void) setKeep:(BOOL)flag {
    _keep = flag;
}




- (void) setTileStatus: (DockTileStatus) status {
    //first see if there is an actual change
    if( _status == status) {
        return;
    }
    
    
    if( status == DTLaunching) {
        //create that time yo!
	NSLog(@"Starting Timer");
        _status = status;
        launchingTimer = [NSTimer scheduledTimerWithTimeInterval: 0.05
                                target: self
                                selector: @selector(launchingTimerFired:)
                                userInfo: nil
                                repeats: YES];
        animationShrinking = YES;
    }
    
    [[self controlView]setNeedsDisplay:YES];
        
    //bring in the new
    _status = status;
}

- (DockTileStatus) tileStatus {
    return _status;
}


- (void) drawWithFrame: (NSRect)cellFrame inView: (NSView*)controlView
{
    NSPoint position = cellFrame.origin;
    NSImage *tileToUse;

    
    if( [controlView isFlipped] ) {
        position.y = [controlView frame].size.height - position.y;
    }
    
    if(_status == DTLaunching) {
        if( [self isHighlighted] ) {
            tileToUse = tileHLImage;
        }
        else {
            tileToUse = tileImage;
        }
    }
    else if(_status == DTLaunched) {
        if( [self isHighlighted] ) {
            tileToUse = tileHLImage;
        }
        else {
            tileToUse = tileImage;
        }
    }
    else if( _status == DTNotRunning) {
        if( [self isHighlighted] ) {
            tileToUse = tileNonRunningHLImage;
        }
        else {
            tileToUse = tileNonRunningImage;
        }
    } 
    else { //the catch all
        //NSLog(@"In DocktileCell drawWithframe Catch all!");
        if( [self isHighlighted] ) {
            tileToUse = tileHLImage;
        }
        else {
            tileToUse = tileImage;
        }
    }
   
    [tileToUse setScalesWhenResized:YES];
    
    [tileToUse setSize: cellFrame.size];
    
   
    [tileToUse compositeToPoint: position
	operation: NSCompositeSourceOver];
    
    [self drawInteriorWithFrame: cellFrame inView: controlView];
}


- (void) drawInteriorWithFrame: (NSRect)cellFrame inView: (NSView *)controlView
{
    NSSize imageSize;
    NSPoint position;
    NSImage *iconImage;

    iconImage = [self image];
    [iconImage setScalesWhenResized:YES];

    //first make the image 75% of the cell size
    imageSize = cellFrame.size;
    imageSize.width  *= iconSize;
    imageSize.height *= iconSize;
    [iconImage setSize: imageSize];
    
    //determine the new position to draw the image
    position = cellFrame.origin;
    position.x = (cellFrame.size.width  * ((1.0 - iconSize)/2)) + cellFrame.origin.x;
    position.y = (cellFrame.size.height * ((1.0 - iconSize)/2)) + cellFrame.origin.y;
    
    if( [controlView isFlipped] ) {
	position.y = [controlView frame].size.height - position.y;
    }
    	
    [iconImage compositeToPoint: position
    operation: NSCompositeSourceOver];
 
}


- (BOOL) trackMouse: (NSEvent*)theEvent
	     inRect: (NSRect)cellFrame
	     ofView: (NSView*)controlView
       untilMouseUp: (BOOL)flag
{
        
    return [super trackMouse: theEvent inRect: cellFrame ofView: controlView untilMouseUp: flag];
}



- (BOOL)startTrackingAt:(NSPoint)startPoint inView:(NSView *)controlView
{
    //NSLog(@"Start Tracking");
    //[self setHighlighted: YES];
    
    [controlView setNeedsDisplay: YES];
    printf("Start Tracking At:\t %f, %f\n", startPoint.x, startPoint.y);
    return YES;
} 

- (BOOL)continueTracking:(NSPoint)lastPoint at:(NSPoint)currentPoint inView:(NSView *)controlView
{
    //NSLog(@"Continue Tracking");
    //NSLog( [self path] );
    [controlView setNeedsDisplay: YES];
    printf("Continue Tracking At:\t %f, %f\n", currentPoint.x, currentPoint.y);
    return YES;
}

- (void)stopTracking:(NSPoint)lastPoint at:(NSPoint)stopPoint inView:(NSView *)controlView mouseIsUp:(BOOL)flag
{
    int row, column;
    
    //NSLog(@"Stop Tracking");
    //[self setHighlighted: NO];
    
    [controlView setNeedsDisplay: YES];
    printf("Stop Tracking At:\t %f, %f\n", stopPoint.x, stopPoint.y);
}

- (void) launchingTimerFired: (NSTimer*)timer {
    NSLog(@"Timer Fired.");
    if( animationShrinking == YES) {
        if( iconSize > 0.5) {
            iconSize -= 0.02;
        }
        else {
            animationShrinking = NO;
            iconSize += 0.02;
        }
    }
    else {
        if( iconSize < 0.75) {
            iconSize += 0.02;
        }
        else {
            if(_status != DTLaunching) {
                [launchingTimer invalidate];
                iconSize = 0.75;
            }
            else {
                animationShrinking = YES;
                iconSize -= 0.02;
            }
        }
    }
    
    [[self controlView] setNeedsDisplay: YES];
    //works great in Cocoa, however in GNUstep the views don't
    //get the message to update unless some kind of input
    //event is received (such as clicking in the window)
}


- (BOOL) doesAcceptFiles:(NSArray*) files {
    NSWorkspace* workspace = [NSWorkspace sharedWorkspace];
    NSEnumerator* enumerator = [files objectEnumerator];
    NSString* pathToCheck;
    NSString* associatedApp;
    NSString* docType;
    
    while( nil != (pathToCheck = [enumerator nextObject])) {
        NSLog(@"");
        NSLog(@"Cells App Path: ");
        NSLog( _path);
        NSLog(@"Path to document: ");
        NSLog(pathToCheck);
        if( YES == [workspace getInfoForFile: pathToCheck application: &associatedApp type: &docType] ) {
            NSLog(@"Workspace says this is a: ");
            NSLog(docType);
            NSLog(@"and it's app is: ");
            NSLog( associatedApp);
            if( [_path isEqualToString: associatedApp] == YES) {
                NSLog(@"We have a match!");
                return YES;
            }
        }
        else {
            NSLog(@"Workspace choked on that one!");
        }
    }
    return NO;
}


- (void) quitDockItemSelected:(id) sender
{
    [NSApp terminate: self];
}

- (void) keepMenuItemSelected:(id) sender 
{
    NSLog(@"DockTileCell:keepMenuItemSelected");
    if( _keep == YES) {
        _keep = NO;
    }
    else {
        _keep = YES;
    }
}

- (void) showLocationItemSelected:(id) sender
{
    [[NSWorkspace sharedWorkspace] openFile: [_path stringByDeletingLastPathComponent]];
}

@end
